#import "iBLE.h"
@import CoreBluetooth;

@interface BlePMDel : NSObject <CBPeripheralManagerDelegate>
@end
@implementation BlePMDel
- (void)peripheralManager:(CBPeripheralManager *)peripheral central:(CBCentral *)central didSubscribeToCharacteristic:(CBCharacteristic *)characteristic {
    ((NSMutableDictionary *)[B4IObjectWrapper getMap:peripheral][@"centrals"])[central.identifier.UUIDString] = central;
    [peripheral setDesiredConnectionLatency:CBPeripheralManagerConnectionLatencyLow forCentral:central];
    [B4IObjectWrapper raiseEvent:peripheral :@"_subscribe:" :@[central.identifier.UUIDString]];
}

- (void)peripheralManager:(CBPeripheralManager *)peripheral central:(CBCentral *)central didUnsubscribeFromCharacteristic:(CBCharacteristic *)characteristic {
    [(NSMutableDictionary *) [B4IObjectWrapper getMap:peripheral][@"centrals"] removeObjectForKey:central.identifier.UUIDString];
    [B4IObjectWrapper raiseEvent:peripheral :@"_unsubscribe:" :@[central.identifier.UUIDString]];
}


- (void)peripheralManagerDidUpdateState:(CBPeripheralManager *)peripheral {
    [B4IObjectWrapper raiseEvent:peripheral :@"_statechanged:" :@[@(peripheral.state)]];
}

- (void)peripheralManager:(CBPeripheralManager *)peripheral didReceiveReadRequest:(CBATTRequest *)request {
    request.value = [[NSMutableData alloc] initWithLength:8];
    NSLog(@"read request");
    [peripheral respondToRequest:request withResult:CBATTErrorSuccess];
}

- (void)peripheralManager:(CBPeripheralManager *)peripheral didReceiveWriteRequests:(NSArray<CBATTRequest *> *)requests {
    CBATTRequest *r = requests[0];
    [peripheral respondToRequest:requests[0] withResult:CBATTErrorSuccess];
    B4IArray *arr = [B4IArray new];
    arr.bytesData = [[NSMutableData alloc] initWithBytes:r.value.bytes length:r.value.length];
    [B4IObjectWrapper raiseEvent:peripheral :@"_newdata:" :@[arr]];
}

@end
@implementation B4IPeripheralManager
#define OBJ ((CBPeripheralManager *) self.object)
+ (Class)getClass {
    return [CBPeripheralManager class];
}
- (int)STATE_POWERED_ON {
    return CBPeripheralManagerStatePoweredOn;
}

- (int)STATE_POWERED_OFF {
    return CBPeripheralManagerStatePoweredOff;
}

- (int)STATE_UNAUTHORIZED {
    return CBPeripheralManagerStateUnauthorized;
}

- (int)STATE_UNSUPPORTED {
    return CBPeripheralManagerStateUnsupported;
}

- (int)STATE_RESETTING {
    return CBPeripheralManagerStateResetting;
}

- (int)STATE_UNKNOWN {
    return CBPeripheralManagerStateUnknown;
}

- (int)State {
    return OBJ.state;
}

- (void)Initialize:(B4I*)bi :(NSString *)EventName {
    BlePMDel *del = [BlePMDel new];
    self.object = [[CBPeripheralManager alloc] initWithDelegate:del queue:nil];
    [B4IObjectWrapper setBIAndEventName:self.object :bi :EventName];
    [B4IObjectWrapper getMap:OBJ][@"delegate"] = del;
    [B4IObjectWrapper getMap:OBJ][@"centrals"] = [NSMutableDictionary new];
}
- (void)Start: (NSString *)Name {
    CBMutableService *service1 = [[CBMutableService alloc] initWithType:[CBUUID UUIDWithString:@"0001"] primary:true];
    CBMutableCharacteristic *char1 = [[CBMutableCharacteristic alloc]
            initWithType:[CBUUID UUIDWithString:@"1001"]
            properties:CBCharacteristicPropertyRead | CBCharacteristicPropertyNotify
                    value:nil permissions:CBAttributePermissionsReadable ];
    CBMutableCharacteristic *char2 = [[CBMutableCharacteristic alloc]
            initWithType:[CBUUID UUIDWithString:@"1002"]
              properties:CBCharacteristicPropertyWrite
                   value:nil permissions:CBAttributePermissionsWriteable ];
    service1.characteristics = @[char1, char2];
    [B4IObjectWrapper getMap:OBJ][@"readChar"] = char1;
    [OBJ addService:service1];
    [OBJ startAdvertising:@{CBAdvertisementDataLocalNameKey: Name, CBAdvertisementDataServiceUUIDsKey: @[service1.UUID]}];
}
- (void)Write:(B4IList *)Centrals :(B4IArray *)Data {
    NSMutableArray *c;
    if (Centrals == nil || Centrals.IsInitialized == false)
        c = nil;
    else {
        c = [NSMutableArray new];
        for (NSString * ce in Centrals.object) {
            CBCentral *central = ((NSDictionary *) ([B4IObjectWrapper getMap:OBJ][@"centrals"]))[ce];
            if (central == nil) {
                NSLog(@"Central not found: %@", ce);
                continue;
            }
            [c addObject:central];
        }
    }
    [OBJ updateValue:Data.bytesData forCharacteristic:[B4IObjectWrapper getMap:OBJ][@"readChar"] onSubscribedCentrals:c];
}
- (void) Stop {
    [OBJ stopAdvertising];
    [OBJ removeAllServices];
}

@end