//
//  iBLE.m
//  iBLE
//
//  Created by b4j on 10/23/14.
//  Copyright (c) 2014 Anywhere Software. All rights reserved.
//

#import "iBLE.h"
#import "iCore.h"
@import CoreBluetooth;

@interface BlePeripheralDel:NSObject <CBPeripheralDelegate>
@end

@implementation BlePeripheralDel {
    __weak CBCentralManager* manager;
}
- (instancetype)init:(CBCentralManager *)manager1 {
    self = [super init];
    self->manager = manager1;
    return self;
}

- (void)peripheral:(CBPeripheral *)peripheral didDiscoverCharacteristicsForService:(CBService *)service error:(NSError *)error {

    int numberToRead = (int)service.characteristics.count;
    for (CBCharacteristic *c in service.characteristics) {
        if ((c.properties & CBCharacteristicPropertyRead) != CBCharacteristicPropertyRead) {
            numberToRead--;
        }
        else {
            [peripheral readValueForCharacteristic:c];
        }
    }
    [B4IObjectWrapper getMap:service][@"numberToRead"] = @(numberToRead);
    if (numberToRead == 0 && service.characteristics.count > 0) {
        [self peripheral:peripheral didUpdateValueForCharacteristic:service.characteristics[0] error:nil];
    }

}

- (void)peripheral:(CBPeripheral *)peripheral didUpdateNotificationStateForCharacteristic:(CBCharacteristic *)characteristic error:(NSError *)error {
    if (error != nil)
        NSLog(@"Error changing notification state for characteristic: %@, error: %@", [[characteristic UUID] description], error);
    else
        NSLog(@"Notification state changed for characteristic: %@", [[characteristic UUID] description]);
}

- (void)peripheral:(CBPeripheral *)peripheral didWriteValueForCharacteristic:(CBCharacteristic *)characteristic error:(NSError *)error {
    if (error != nil)
        NSLog(@"Error writing value for characteristic: %@, error: %@", [[characteristic UUID] description], error);
    [B4IObjectWrapper raiseEvent:manager :@"_writecomplete::" :@[characteristic.UUID.UUIDString, @(error == nil)]];
}


- (void)peripheral:(CBPeripheral *)peripheral didUpdateValueForCharacteristic:(CBCharacteristic *)characteristic error:(NSError *)error {
    CBService *service = characteristic.service;
    int numberToRead = [[B4IObjectWrapper getMap:service][@"numberToRead"] intValue];
    numberToRead--;
    [B4IObjectWrapper getMap:service][@"numberToRead"] = @(numberToRead);
    if (numberToRead <= 0) {
        B4IMap *map = [B4IMap new];
        [map Initialize];
        for (CBCharacteristic *c in service.characteristics) {
            B4IArray *buffer = [B4IArray new];
            NSData *value = c.value;
            if (value == nil)
                buffer = [buffer initBytes:@[@0]];
            else
                buffer.bytesData = value;
            [map Put:[c.UUID description] :buffer];
        }
        [B4IObjectWrapper getMap:manager][@"last_updated_char"] = [characteristic.UUID description];
        [B4IObjectWrapper raiseEvent:manager :@"_dataavailable::" :@[service.UUID.UUIDString, map]];
    }
}


- (void)peripheral:(CBPeripheral *)peripheral didDiscoverServices:(NSError *)error {

    if (error != nil) {
        NSLog(@"Error discovering services: %@", error);
        [B4IObjectWrapper raiseEvent:manager :@"_disconnected" :nil];
    } else {
        NSLog(@"Services discovery completed.");
        B4IList *services = [B4IList new];
        [services Initialize];
        for (CBService *ser in peripheral.services) {
            [services Add:ser.UUID.UUIDString];
        }
        [B4IObjectWrapper getMap:manager][@"connected"] = peripheral;
        [B4IObjectWrapper raiseEvent:manager :@"_connected:" :@[services]];

    }

}

@end

@interface BleDelegate : NSObject <CBCentralManagerDelegate>
@property (nonatomic) NSMutableDictionary *discoveredDevices;
@end

@implementation BleDelegate

- (instancetype)init {
    self = [super init];
    self.discoveredDevices = [NSMutableDictionary new];
    return self;
}
- (void)centralManagerDidUpdateState:(CBCentralManager *)central {
    [B4IObjectWrapper raiseEvent:central :@"_statechanged:" :@[@(central.state)]];
}

- (void)centralManager:(CBCentralManager *)central didDiscoverPeripheral:(CBPeripheral *)peripheral advertisementData:(NSDictionary *)advertisementData RSSI:(NSNumber *)RSSI {
    B4I *bi = [B4I shared];
    NSString *id = [bi returnString:peripheral.identifier.UUIDString];
    self.discoveredDevices[id] = peripheral;
    B4IMap *map = [B4IMap new];
    map.dict = advertisementData;
    if (RSSI == nil)
        RSSI = @(0);
    [B4IObjectWrapper raiseEvent:central :@"_devicefound::::" :@[[bi returnString:peripheral.name],
            id, map, RSSI]];
}

- (void)centralManager:(CBCentralManager *)central didConnectPeripheral:(CBPeripheral *)peripheral {
    BlePeripheralDel* del = [[BlePeripheralDel alloc] init:central];
    [B4IObjectWrapper getMap:peripheral][@"delegate"] = del;
    peripheral.delegate = del;
    NSLog(@"Discovering services");
    if (peripheral.services == nil)
        [peripheral discoverServices:nil];
    else
        [del peripheral:peripheral didDiscoverServices:nil];

}

- (void)centralManager:(CBCentralManager *)central didFailToConnectPeripheral:(CBPeripheral *)peripheral error:(NSError *)error {
    [[B4I shared] setLastError:error];
    [B4IObjectWrapper raiseEvent:central :@"_disconnected" :nil];
}

- (void)centralManager:(CBCentralManager *)central didDisconnectPeripheral:(CBPeripheral *)peripheral error:(NSError *)error {
    [[B4I shared] setLastError:error];
    [B4IObjectWrapper raiseEvent:central :@"_disconnected" :nil];
}

- (void)centralManager:(CBCentralManager *)central didRetrieveConnectedPeripherals:(NSArray *)peripherals {

}

- (void)centralManager:(CBCentralManager *)central didRetrievePeripherals:(NSArray *)peripherals {

}


@end
@implementation BleManager
#define OBJ ((CBCentralManager *) self.object)

+ (Class)getClass {
    return [CBCentralManager class];
}
- (int)STATE_POWERED_ON {
    return CBCentralManagerStatePoweredOn;
}

- (int)State {
    return OBJ.state;
}


- (int)STATE_POWERED_OFF {
    return CBCentralManagerStatePoweredOff;
}

- (int)STATE_UNAUTHORIZED {
    return CBCentralManagerStateUnauthorized;
}

- (int)STATE_UNSUPPORTED {
    return CBCentralManagerStateUnsupported;
}

- (int)STATE_RESETTING {
    return CBCentralManagerStateResetting;
}

- (int)STATE_UNKNOWN {
    return CBCentralManagerStateUnknown;
}
- (void)ReadData:(NSString *)Service {
    CBPeripheral *per = [B4IObjectWrapper getMap:OBJ][@"connected"];
    CBService *ser = [self getService:Service];
    if (ser.characteristics == nil)
        [per discoverCharacteristics:nil forService:ser];
    else
        [per.delegate peripheral:per didDiscoverCharacteristicsForService:ser error:nil];
}
- (void)ReadData2:(NSString*)Service :(NSString *)Characteristic {
    CBPeripheral *per = [B4IObjectWrapper getMap:OBJ][@"connected"];
    CBService *ser = [self getService:Service];
    CBCharacteristic *c = [self getChar:ser :Characteristic];
    if((c.properties & CBCharacteristicPropertyRead) != CBCharacteristicPropertyRead)
        [NSException raise:@"" format:@"Characteristic %@ is not readable.", [c.UUID description]];
    [per readValueForCharacteristic:c];
}
- (void)SetNotify:(NSString *)Service :(NSString *)Characteristic :(BOOL)Notify {
    CBPeripheral *per = [B4IObjectWrapper getMap:OBJ][@"connected"];
    CBService *ser = [self getService:Service];
    CBCharacteristic *c = [self getChar:ser :Characteristic];
    [per setNotifyValue:Notify forCharacteristic:c];
}
- (void)Initialize:(B4I *)bi :(NSString*)EventName {
    BleDelegate *del = [BleDelegate new];
    NSDictionary *options = nil;
    self.object = [[CBCentralManager alloc] initWithDelegate:del
                                                   queue:nil options:options];
    [B4IObjectWrapper setBIAndEventName:self.object :bi :EventName];
    [B4IObjectWrapper getMap:OBJ][@"delegate"] = del;
}
- (void)Scan:(B4IList *)ServiceUUIDs {
    [self Scan2:ServiceUUIDs :false];
}
- (void)Scan2:(B4IList *)ServiceUUIDs :(bool) AllowDuplicates{
    NSMutableArray *uuids = nil;
    if (ServiceUUIDs != nil && ServiceUUIDs.IsInitialized) {
        uuids = [NSMutableArray new];
        for (NSString *u in ServiceUUIDs.object) {
            [uuids addObject:[CBUUID UUIDWithString:u]];
        }
    }
    NSMutableDictionary *dic = [NSMutableDictionary new];
    dic[CBCentralManagerScanOptionAllowDuplicatesKey] = @(AllowDuplicates);
    [OBJ scanForPeripheralsWithServices:uuids options:dic];
}
- (CBCharacteristic *)getChar:(CBService *)ser :(NSString *)Characteristic{
    CBCharacteristic *c = nil;
    if (ser.characteristics.count == 0)
        [NSException raise:@"" format:@"You must first call ReadData and wait for DataAvailable event."];
    for (CBCharacteristic *chr in ser.characteristics) {
        if ([[chr.UUID description] isEqualToString:Characteristic]) {
            c = chr;
            break;
        }
    }
    if (c == nil)
        [NSException raise:@"" format:@"Characteristic not found: %@", Characteristic];
    return c;
}

- (void) StopScan {
    [OBJ stopScan];
}
- (void)Connect:(NSString *)DeviceId {
    BleDelegate *del = OBJ.delegate;
    CBPeripheral *p = del.discoveredDevices[DeviceId];
    [B4IObjectWrapper getMap:OBJ][@"start_connection"] = p;
    [OBJ cancelPeripheralConnection:p];
    [OBJ connectPeripheral:p options:nil];
}
-(void)Disconnect {
    CBPeripheral *per = [B4IObjectWrapper getMap:OBJ][@"start_connection"];
    if (per != nil) {
        [OBJ cancelPeripheralConnection:per];
        [[B4IObjectWrapper getMap:OBJ] removeObjectForKey:@"start_connection"];
    }
}

- (CBService *)getService:(NSString *)Service {
    CBPeripheral *per = [B4IObjectWrapper getMap:OBJ][@"connected"];
    if (per == nil)
        [NSException raise:@"" format:@"No device connected"];
    CBService *ser;
    for (CBService *s in per.services) {
        if ([s.UUID.UUIDString isEqualToString:Service])
            ser = s;
    }
    if (ser == nil)
        [NSException raise:@"" format:@"Service not found"];
    return ser;
}
- (void)WriteData:(NSString *)Service :(NSString *)Characteristic :(B4IArray *)Data {
    [self writeData:Service :Characteristic :Data :CBCharacteristicWriteWithoutResponse];
}
- (void)WriteDataWithResponse:(NSString *)Service :(NSString *)Characteristic :(B4IArray *)Data {
    [self writeData:Service :Characteristic :Data :CBCharacteristicWriteWithResponse];
}
- (int)GetCharacteristicProperties:(NSString *)Service :(NSString *)Characteristic {
    CBService *ser = [self getService:Service];
    CBCharacteristic *c = [self getChar:ser :Characteristic];
    return c.properties;
}
- (void)writeData:(NSString *)Service :(NSString *)Characteristic :(B4IArray *)Data :(CBCharacteristicWriteType)writeType{
    CBPeripheral *per = [B4IObjectWrapper getMap:OBJ][@"connected"];
    CBService *ser = [self getService:Service];
    CBCharacteristic *c = [self getChar:ser :Characteristic];
    [per writeValue:Data.bytesData forCharacteristic:c type:writeType];
}
- (NSObject *)GetPeripheralObject {
    return [B4IObjectWrapper getMap:OBJ][@"connected"];
}
- (NSString *)LastUpdatedCharacteristic {
    return [[B4I shared] returnString:[B4IObjectWrapper getMap:OBJ][@"last_updated_char"]];
}
- (NSObject *)GetServiceObject:(NSString *)Service {
    return [self getService:Service];
}
- (NSObject *)GetCharacteristicObject:(NSString *)ServiceId :(NSString *)CharacteristicId {
    CBService *ser = [self getService:ServiceId];
    CBCharacteristic *c = [self getChar:ser :CharacteristicId];
    return c;
}

@end

